<?php

namespace Drupal\fontawesome_ui;

use Drupal\Core\Extension\ExtensionPathResolver;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Yaml;

/**
 * Provides manager functions for Font Awesome metadata.
 */
class FontAwesomeMetadataManager implements FontAwesomeMetadataManagerInterface {

  /**
   * The extension path resolver.
   *
   * @var \Drupal\Core\Extension\ExtensionPathResolver
   */
  protected $extensionPathResolver;

  /**
   * Constructs a new FontAwesomeMetadataManager object.
   *
   * @param \Drupal\Core\Extension\ExtensionPathResolver $extension_path_resolver
   *   The extension path resolver.
   */
  public function __construct(ExtensionPathResolver $extension_path_resolver) {
    $this->extensionPathResolver = $extension_path_resolver;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('extension.path.resolver')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getCategories($version_type = 'free', $category_name = 'all') {
    $version_type = strtolower($version_type);
    $module_path = $this->extensionPathResolver->getPath('module', 'fontawesome_ui');
    $file_path = $module_path . '/metadata/categories.' . ($version_type == 'pro' ? 'pro' : 'free') . '.yml';
    $categories = [];

    if (file_exists($file_path)) {
      $content = file_get_contents($file_path);
      $data = Yaml::parse($content);

      if ($category_name !== 'all' && !empty($category_name)) {
        $category_name_lower = strtolower(trim($category_name));

        foreach ($data as $category) {
          $label = strtolower($category['label']);
          if ($label === $category_name_lower) {
            $categories[$category['label']] = $category['icons'];
            return $categories;
          }
        }
        return [];
      }

      foreach ($data as $category) {
        $label = $category['label'];
        $icons = $category['icons'];
        $categories[$label] = $icons;
      }
    }
    return $categories;
  }

  /**
   * {@inheritdoc}
   */
  public function getIcons($version_type = 'free', $category_name = 'all', array $styles = [], $class_prefix = FALSE, $icon_name = '') {
    $version_type = strtolower($version_type);
    $module_path = $this->extensionPathResolver->getPath('module', 'fontawesome_ui');
    $file_path = $module_path . '/metadata/icons.' . ($version_type == 'pro' ? 'pro' : 'free') . '.yml';
    $icons = [];

    if (file_exists($file_path)) {
      $content = file_get_contents($file_path);
      $data = Yaml::parse($content);

      if (isset($data) && is_array($data)) {
        if ($category_name !== 'all' && !empty($category_name)) {
          $categories = $this->getCategories($version_type, $category_name);
          if (!empty($categories)) {
            $category_icons = reset($categories);
            $data = array_intersect_key($data, array_flip($category_icons));
          }
          else {
            return [];
          }
        }

        if (empty($styles)) {
          $styles = $this->getIconStyles($version_type, 6);
        }
        $styles_set = array_flip($styles);

        $icons = array_reduce(array_keys($data), function ($carry, $icon) use ($data, $styles_set, $class_prefix, $icon_name) {
          if (!empty($icon_name) && $icon !== $icon_name) {
            return $carry;
          }

          $icon_styles = $data[$icon]['styles'] ?? [];
          foreach ($icon_styles as $style) {
            if (isset($styles_set[$style])) {
              $icon_key = $icon;
              if ($class_prefix) {
                $icon_key = 'fa-' . $style . ' fa-' . $icon;
              }
              $carry[$icon_key] = $icon_key;
            }
          }

          return $carry;
        }, []);
      }
    }
    return $icons;
  }

  /**
   * {@inheritdoc}
   */
  public function getCategorizedIcons($version, $version_type = 'free', $category_name = 'all', array $styles = [], $icon_name = '') {
    // Convert version_type to lowercase to ensure case-insensitive comparison.
    $version_type = strtolower($version_type);

    // Validate version_type.
    if (!in_array($version_type, ['free', 'pro'])) {
      return [];
    }

    // Get the path to the module.
    $module_path = $this->extensionPathResolver->getPath('module', 'fontawesome_ui');

    // Determine the categories and icon families file paths based on the
    // version type.
    $categories_path = $module_path . '/metadata/categories.' . $version_type . '.yml';
    $icon_families_path = $module_path . '/metadata/icon-families.' . $version_type . '.yml';

    // Initialize an empty array to store categorized icons.
    $categorized_icons = [];

    // Check if the files exist.
    if (file_exists($categories_path) && file_exists($icon_families_path)) {
      // Read and parse the contents of the categories and icon families files.
      $categories = Yaml::parse(file_get_contents($categories_path));
      $icon_families = Yaml::parse(file_get_contents($icon_families_path));

      // Check if the parsed data is valid arrays.
      if (is_array($categories) && is_array($icon_families)) {
        // If styles array is empty, get all available styles.
        if (empty($styles)) {
          $styles = fontawesome_ui_icon_styles($version_type, intval($version));
        }

        // Create a set of styles for quick lookup.
        $styles_set = array_flip($styles);

        // Filter icons by version and version type.
        foreach ($categories as $category => $details) {
          if ($category_name !== 'all' && $category_name !== $category) {
            continue;
          }

          // Iterate over each icon in the category.
          foreach ($details['icons'] as $icon) {
            // If a specific icon name is provided, skip other icons.
            if (!empty($icon_name) && $icon !== $icon_name) {
              continue;
            }

            if (isset($icon_families[$icon])) {
              // Check the changes to see if the icon is available in the given
              // version.
              $min_version = intval(str_replace('.', '', min($icon_families[$icon]['changes'])));
              if ($min_version > intval(str_replace('.', '', $version))) {
                continue;
              }

              // List of free font styles for icon.
              $free_styles = [];
              if (isset($icon_families[$icon]['familyStylesByLicense']['free'])) {
                foreach ($icon_families[$icon]['familyStylesByLicense']['free'] as $free_detail) {
                  $free_styles[] = $free_detail['style'];
                }
              }

              // Check if the icon is available in the specified
              // version type and styles.
              $icon_styles = [];
              if (isset($icon_families[$icon]['familyStylesByLicense'][$version_type])) {
                foreach ($icon_families[$icon]['familyStylesByLicense'][$version_type] as $style_detail) {
                  if (isset($styles_set[$style_detail['style']])) {
                    switch ($style_detail['family']) {
                      case 'sharp':
                        $font_style = $style_detail['style'];
                        $font_family = 'sharp';
                        break;

                      case 'duotone':
                        $font_style = 'duotone';
                        $font_family = 'classic';
                        break;

                      default:
                        $font_style = $style_detail['style'];
                        $font_family = $style_detail['style'] == 'brands' ? 'brands' : 'classic';
                        break;
                    }

                    // Make icon classes by font name and style.
                    $sharp = $style_detail['family'] == 'sharp' ? 'fa-sharp ' : '';
                    $duotone = $style_detail['family'] == 'duotone' ? 'fa-duotone ' : '';
                    $icon_class = $sharp . $duotone . 'fa-' . $style_detail['style'] . ' fa-' . $icon;

                    // Mark icon styles and family by license.
                    $icon_type = in_array($style_detail['style'], $free_styles) ? 'free' : 'pro';

                    // Create icon style details.
                    $icon_styles[] = [
                      'family' => $font_family,
                      'style' => $font_style,
                      'class' => $icon_class,
                      'type' => $icon_type,
                    ];
                  }
                }
              }

              // Check if style is not empty to make sure everything is fine.
              if (!empty($icon_styles)) {
                if (!isset($categorized_icons[$details['label']])) {
                  $categorized_icons[$details['label']] = [];
                }

                // Add icon information for search.
                $search_terms = array_merge([$icon], $icon_families[$icon]['aliases']['names'] ?? [], $icon_families[$icon]['search']['terms'] ?? []);
                $search_terms = implode(' ', array_map('strtolower', $search_terms));

                // Add icon information to its category.
                $categorized_icons[$details['label']][$icon] = [
                  'label' => $icon_families[$icon]['label'] ?? '',
                  'search_terms' => $search_terms,
                  'icons' => $icon_styles,
                  'free' => $free_styles,
                ];
              }
            }
          }
        }

        // Get uncategorized icons and merge to categorized icons.
        $uncategorized_icons = $this->getUncategorizedIcons($version, $version_type, $styles);
        $categorized_icons = array_merge($categorized_icons, $uncategorized_icons);
      }
    }

    // Return the array of categorized icons.
    return $categorized_icons;
  }

  /**
   * {@inheritdoc}
   */
  public function getUncategorizedIcons($version, $version_type = 'free', $styles = []) {
    // Convert version_type to lowercase to ensure case-insensitive comparison.
    $version_type = strtolower($version_type);

    // Validate version_type.
    if (!in_array($version_type, ['free', 'pro'])) {
      return [];
    }

    // Get the path to the module.
    $module_path = $this->extensionPathResolver->getPath('module', 'fontawesome_ui');

    // Determine the file paths.
    $icon_families_path = $module_path . '/metadata/icon-families.' . $version_type . '.yml';
    $categories_path = $module_path . '/metadata/categories.' . $version_type . '.yml';
    $sponsors_path = $module_path . '/metadata/sponsors.yml';

    // Initialize an array to store uncategorized icons.
    $uncategorized_icons = [
      'Brands' => [],
      'Sponsors' => [],
      'Uncategorized' => [],
    ];

    // Check if the files exist.
    if (file_exists($icon_families_path) && file_exists($categories_path) && file_exists($sponsors_path)) {
      // Read and parse the contents of the icon
      // families, categories, and sponsors files.
      $icon_families = Yaml::parse(file_get_contents($icon_families_path));
      $categories = Yaml::parse(file_get_contents($categories_path));
      $sponsors = Yaml::parse(file_get_contents($sponsors_path));

      // Extract all icons from each category and remove duplicates.
      $all_icons = array_merge_recursive(...array_column($categories, 'icons'));
      $unique_icons = array_values(array_unique($all_icons));

      // List of all categorized icons for checking
      // icons not assigned to any categories.
      $categorized_icons = array_combine($unique_icons, $unique_icons);

      // Extract all sponsor icons.
      $sponsor_icons_list = [];
      foreach ($sponsors as $sponsor) {
        if (isset($sponsor['icons'])) {
          $sponsor_icons_list = array_merge($sponsor_icons_list, $sponsor['icons']);
        }
      }

      // If styles array is empty, get all available styles.
      if (empty($styles)) {
        $styles = fontawesome_ui_icon_styles($version_type, intval($version));
      }

      // Create a set of styles for quick lookup.
      $styles_set = array_flip($styles);

      // Filter icons by version and version type.
      foreach ($icon_families as $icon => $details) {
        // Check the changes to see the icon is available in the given version.
        $min_version = intval(str_replace('.', '', min($details['changes'])));
        if ($min_version > intval(str_replace('.', '', $version))) {
          continue;
        }

        // Determine the styles and family of the icon.
        $icon_styles = [];
        $free_styles = [];
        $is_sponsor = in_array($icon, $sponsor_icons_list);
        $is_brand = FALSE;

        // Determine if the style is free.
        if (isset($details['familyStylesByLicense']['free'])) {
          foreach ($details['familyStylesByLicense']['free'] as $free_detail) {
            $free_styles[] = $free_detail['style'];
          }
        }

        // Check if the icon is available in
        // the specified version type and styles.
        if (isset($details['familyStylesByLicense'][$version_type])) {
          foreach ($details['familyStylesByLicense'][$version_type] as $style_detail) {
            if (isset($styles_set[$style_detail['style']])) {
              if ($style_detail['style'] === 'brands') {
                $is_brand = TRUE;
              }

              // Determine the font family.
              $font_family = ($style_detail['family'] == 'sharp') ? 'sharp' : 'classic';

              // Make icon classes by font name and style.
              $sharp = $style_detail['family'] == 'sharp' ? 'fa-sharp ' : '';
              $duotone = $style_detail['family'] == 'duotone' ? 'fa-duotone ' : '';
              $icon_class = $sharp . $duotone . 'fa-' . $style_detail['style'] . ' fa-' . $icon;

              // Mark icon styles and family by license.
              $icon_type = in_array($style_detail['style'], $free_styles) ? 'free' : 'pro';

              // Create icon style details.
              $icon_styles[] = [
                'family' => $font_family,
                'style' => $style_detail['style'],
                'class' => $icon_class,
                'type' => $icon_type,
              ];
            }
          }
        }

        // Categorize the icon based on its type.
        if ($is_brand) {
          $name = 'Brands';
        }
        elseif ($is_sponsor) {
          $name = 'Sponsors';
        }
        else {
          $name = 'Uncategorized';
        }

        // Determine if icon is not assigned to any category.
        if (!isset($categorized_icons[$icon])) {
          // Add icon information for search.
          $search_terms = array_merge([$icon], $details['aliases']['names'] ?? [], $details['search']['terms'] ?? []);
          $search_terms = implode(' ', array_map('strtolower', $search_terms));

          // Add icon information to its category.
          $uncategorized_icons[$name][$icon] = [
            'label' => $details['label'] ?? '',
            // 'aliases' => $details['aliases']['names'] ?? [],
            'search_terms' => $search_terms,
            'icons' => $icon_styles,
            'free' => $free_styles,
          ];
        }
      }
    }

    // Return the array of uncategorized icons.
    return $uncategorized_icons;
  }

}
