<?php

namespace Drupal\fontawesome_ui;

/**
 * Interface for FontAwesomeMetadataManager.
 */
interface FontAwesomeMetadataManagerInterface {

  /**
   * Retrieves the list of Font Awesome icon categories.
   *
   * This function returns categories with their respective icons, or a specific
   * category if requested.
   *
   * @param string $version_type
   *   The version type of Font Awesome icons, either 'free' or 'pro'. Default
   *   is 'free'.
   * @param string $category_name
   *   The name of the category to retrieve. Default is 'all'.
   *
   * @return array
   *   An array of categories with their respective icons. If a specific
   *   category is requested, only that category and its icons are returned.
   */
  public function getCategories($version_type = 'free', $category_name = 'all');

  /**
   * Retrieves the list of Font Awesome all icons.
   *
   * This function allows filtering by version type, category, styles, class
   * prefix, and icon name.
   *
   * @param string $version_type
   *   The version type of Font Awesome icons, either 'free' or 'pro'. Default
   *   is 'free'.
   * @param string $category_name
   *   The name of the category to retrieve. Default is 'all'.
   * @param array $styles
   *   An array of styles to filter icons by. Default is an empty
   *   array (all styles).
   * @param bool $class_prefix
   *   Whether to add the "fa-" prefix to icon names. Default is FALSE.
   * @param string $icon_name
   *   A specific icon name to retrieve. Default is an empty string.
   *
   * @return array
   *   An array of icons where the key and value are both the icon name,
   *   optionally including style and class prefix.
   */
  public function getIcons($version_type = 'free', $category_name = 'all', array $styles = [], $class_prefix = FALSE, $icon_name = '');

  /**
   * Retrieves categorized Font Awesome icons based on specified parameters.
   *
   * This function allows filtering by version, version type, category,
   * styles, and icon name.
   *
   * @param string $version
   *   The version of Font Awesome icons (e.g., 5, 6).
   * @param string $version_type
   *   The version type of Font Awesome icons, either 'free' or 'pro'.
   *   Default is 'free'.
   * @param string $category_name
   *   The name of the category to retrieve. Default is 'all'.
   * @param array $styles
   *   An array of styles to filter icons by. Default is an empty
   *   array (all styles).
   * @param string $icon_name
   *   A specific icon name to retrieve. Default is an empty string.
   *
   * @return array
   *   An array of categorized icons.
   */
  public function getCategorizedIcons($version, $version_type = 'free', $category_name = 'all', array $styles = [], $icon_name = '');

  /**
   * Retrieves uncategorized Font Awesome icons and categorizes them.
   *
   * Icons are categorized into Brands, Sponsors, and Uncategorized.
   *
   * @param string $version
   *   The version of Font Awesome icons (e.g., 5, 6).
   * @param string $version_type
   *   The version type of Font Awesome icons, either 'free' or 'pro'.
   *   Default is 'free'.
   * @param array $styles
   *   An array of styles to filter icons by. Default is an empty
   *   array (all styles).
   *
   * @return array
   *   An array of categorized icons.
   */
  public function getUncategorizedIcons($version, $version_type = 'free', array $styles = []);

}
